from publishers import publisher
from subscribers import subscriber
from requestors import requestor
from repliers import replier
from constants import *

"""
Please run the following import to get started::

    from zmqwrapper import *

You will see the following utility methods that can be used to create
the entities

- publisher
- subcriber
- requestor
- replier

Please refer individual package docs for more details.
"""




import zmq
from sockets import ClientConnection
from constants import *
import threading

def subscriber(address,topics,callback,message_type):
    """
    Creates a subscriber binding to the given address and 
    subscribe the given topics.
    The callback is invoked for every message received.

        Args:
            - address: the address to bind the PUB socket to.
            - topics: the topics to subscribe
            - callback: the callback to invoke for every message. Must accept 2 variables - topic and message
            - message_type: the type of message to receive
    """
    return Subscriber(address,topics,callback,message_type)
    
class Subscriber(ClientConnection):
    """
    Subscriber that can read messages from ZMQ
    
        Args:
            - address: the address to bind to
            - topics: the topics to subscribe
            - callback: the callback to invoke for every message
            - message_type: the type of message to receive
    """
        
    def __init__(self,address,topics,callback,message_type):
        self._active = True
        self._topics = topics
        self._callback = callback
        self._message_type = message_type
        super(Subscriber,self).__init__(address,zmq.SUB)
        for topic in self._topics:
            self._sock.setsockopt(zmq.SUBSCRIBE,topic)
        
                    
    def _consume(self):
        while self._active:
            try:
                topic, message=super(Subscriber,self).receive(self._message_type)
                #process the message
                self._callback(topic,message)
            except zmq.ZMQError:
                pass
    
    
    def start(self):
        """
        Start a thread that consumes the messages and invokes the callback
        """
        t=threading.Thread(target=self._consume)
        t.start()
        
    def stop(self):
        """
        Stop the consumer thread
        """
        self._active = False



from publishers import publisher
from subscribers import subscriber
from constants import *
import time
import threading

def test_init_publisher():
    p=publisher('ipc:///tmp/test/0')
    assert p is not None
    assert p.sock() is not None
    p.close()
    
def test_init_subscriber():
    p=publisher('ipc:///tmp/test/0')
    def foo(topic, message):
        pass
    s=subscriber('ipc:///tmp/test/0',[''],foo,JSON)
    assert p is not None
    assert p.sock() is not None
    assert s is not None
    assert s.sock() is not None
    p.close()
    s.close()    
    
def test_init_basic_subscribe():
    m = threading.Condition()
    m.acquire()
    p=publisher('ipc:///tmp/test/0')
    def foo(topic, message):
        m.acquire()
        assert message == 'test message'
        m.notifyAll()
        m.release()
    s=subscriber('ipc:///tmp/test/0',['test'],foo,STRING)
    s.start()
    assert p is not None
    assert p.sock() is not None
    assert s is not None
    assert s.sock() is not None
    p.publish('test message',STRING,'test')
    m.wait() 
    m.release()
    s.stop()   
    s.close()        
    p.close()




#message types
"""
The message types that are supported by the package  are - 

- RAW
- PYOBJ
- JSON
- MULTIPART
- STRING
- UNICODE

This map to the same types supported by pyzmq
"""
RAW, PYOBJ, JSON, MULTIPART, STRING, UNICODE = range(6)



from requestors import requestor
from repliers import replier
from constants import *
import time
import threading 

def test_init_requestor():
    def foo(message,requestor):
        pass
    rq=requestor('ipc:///tmp/test/0',foo,JSON)
    assert rq is not None
    assert rq.sock() is not None
    rq.close()
    
def test_init_replier():
    def foo(message,requestor):
        pass
    rq=requestor('ipc:///tmp/test/0',foo,JSON)
    rp=replier('ipc:///tmp/test/0',foo,JSON)
    assert rq is not None
    assert rq.sock() is not None
    assert rp is not None
    assert rp.sock() is not None
    rq.close()
    rp.close()    
    
def test_init_basic_reqrep():
    m = threading.Condition()
    m.acquire()
    def foo(message,requestor):
        m.acquire()
        assert message == 'test message'
        m.notifyAll()
        m.release()
    def foo1(message,replier):
        m.acquire()
        assert message == 'test message'
        replier.reply(message,JSON)
        m.notifyAll()
        m.release()
    rq=requestor('ipc:///tmp/test/0',foo,JSON)
    rp=replier('ipc:///tmp/test/0',foo1,JSON)
    rq.start()
    rp.start()
    assert rq is not None
    assert rq.sock() is not None
    assert rp is not None
    assert rp.sock() is not None
    rq.request('test message',JSON)
    m.wait()  
    m.release()  
    rq.stop()
    rp.stop()
    rq.close()
    rp.close()



import zmq
from sockets import ServerConnection
from constants import *
import threading

def replier(address,callback,message_type):
    """
    Creates a replier binding to the given address send replies.
    The callback is invoked for every request received.

        Args:
            - address: the address to bind the REP socket to.
            - callback: the callback to invoke for every message. Must accept 2 variables - message and the replier
            - message_type: the type of message to receive
    """
    return Replier(address,callback,message_type)
    
class Replier(ServerConnection):
    """
    Requestor that that can respond to requests of given type
    
        Args:
            - address: the address to bind to
            - callback: the callback to invoke for every request
            - message_type: the type of reply to send
    """
        
    def __init__(self,address,callback,message_type):
        self._active = True
        self._callback = callback
        self._message_type = message_type
        super(Replier,self).__init__(address,zmq.REP)
        
                    
    def _consume(self):
        while self._active:
            try:
                topic, message=super(Replier,self).receive(self._message_type)
                #process the message
                self._callback(message,self)
            except zmq.ZMQError:
                pass
    
    
    def start(self):
        """
        Start a thread that consumes the requests and invokes the callback
        """
        t=threading.Thread(target=self._consume)
        t.start()
        
    def stop(self):
        """
        Stop the consumer thread
        """
        self._active = False
        
    def reply(self,message,message_type):
        """
        Send a reply message of the given type
        
        Args:
            - message: the message to publish
            - message_type: the type of message being sent
        """
        if message_type == MULTIPART:
            raise Exception("Unsupported reply type")
            
        super(Replier,self).send(message,message_type)
    



#!/usr/bin/env python

from distutils.core import setup

setup(name='pyzmq-wrapper',
      version='0.1.1b',
      description='Wrapper classes for pyzmq',
      long_description='Please visit https://github.com/supercoderz/pyzmq-wrapper for more details',
      author='Narahari Allamraju',
      author_email='anarahari@gmail.com',
      url='https://github.com/supercoderz/pyzmq-wrapper',
      packages=['zmqwrapper'],
      requires=['pyzmq','pytest'],
     )



import zmq
from sockets import ServerConnection
from constants import *

def publisher(address):
    """
    Creates a publisher binding to the given port number.

        Args:
            - address: the address to bind the PUB socket to.
    """
    return Publisher(address)
    
class Publisher(ServerConnection):
    """
    Publisher that can send messages to ZMQ
    
        Args:
            - address: the address to bind to
    """

    def __init__(self,address):
        super(Publisher,self).__init__(address,zmq.PUB)
        
        
    def publish(self,message,message_type,topic=''):
        """
        Publish the message on the PUB socket with the given topic name.
        
        Args:
            - message: the message to publish
            - message_type: the type of message being sent
            - topic: the topic on which to send the message. Defaults to ''.
        """
        super(Publisher,self).send(message,message_type,topic)




import zmq
from constants import *


class SendReceiveMixin:
    """
    Provides send or receive functionality for the sockets
    """
    
    def send(self,message,message_type,topic=''):
        """
        Send the message on the socket.
        
        Args:
            - message: the message to publish
            - message_type: the type of message being sent
            - topic: the topic on which to send the message. Defaults to ''.
        """
        if message_type == RAW:
            self._sock.send(message)
        elif message_type == PYOBJ:
            self._sock.send_pyobj(message)
        elif message_type == JSON:
            self._sock.send_json(message)
        elif message_type == MULTIPART:
            self._sock.send_multipart([topic, message])
        elif message_type == STRING:
            self._sock.send_string(message)
        elif message_type == UNICODE:
            self._sock.send_unicode(message)
        else:
            raise Exception("Unknown message type %s"%(message_type,))
            
    def receive(self,message_type):
        """
        Receive the message of the specified type and retun
        
            Args:
                - message_type: the type of the message to receive
                
            Returns:
                - the topic of the message
                - the message received from the socket
        """
        topic = None
        message = None
        if message_type == RAW:
            message = self._sock.recv(flags=zmq.NOBLOCK)
        elif message_type == PYOBJ:
            message = self._sock.recv_pyobj(flags=zmq.NOBLOCK)
        elif message_type == JSON:
            message = self._sock.recv_json(flags=zmq.NOBLOCK)
        elif message_type == MULTIPART:
            data = self._sock.recv_multipart(flags=zmq.NOBLOCK)
            message = data[1]
            topic = data[0]
        elif message_type == STRING:
            message = self._sock.recv_string(flags=zmq.NOBLOCK)
        elif message_type == UNICODE:
            message = self._sock.recv_unicode(flags=zmq.NOBLOCK)
        else:
            raise Exception("Unknown message type %s"%(self._message_type,))
            
        return (topic, message)


class ServerConnection(object,SendReceiveMixin):
    """
    Creates a server side socket of given type.
    
        Args:
            - address: the address to use
            - socket_type: the tyoe of socket to open
    """

    _address = None
    _ctx = None
    _sock = None
    
    def __init__(self, address,socket_type):
        self._address = address
        self._ctx = zmq.Context()
        self._sock = self._ctx.socket(socket_type)
        self._sock.bind(address)               

    def sock(self):
        """
        Returns the zmq socket object being used for the connection. This can be used
        to send messages with additional flags etc.
        
            Returns:
                - The underlying zmq socket
        """
        return self._sock
        
    def close(self):
        """
        Close the socket connection.
        """
        self._sock.close()

class ClientConnection(object,SendReceiveMixin):
    """
    Creates a client side socket of given type.
    
        Args:
            - address: the address to use
            - socket_type: the tyoe of socket to open
    """

    _address = None
    _ctx = None
    _sock = None
    
    def __init__(self, address,socket_type):
        self._address = address
        self._ctx = zmq.Context()
        self._sock = self._ctx.socket(socket_type)
        self._sock.connect(address)               

    def sock(self):
        """
        Returns the zmq socket object being used for the connection. This can be used
        to receive messages with additional flags etc.
        
            Returns:
                - The underlying zmq socket
        """
        return self._sock

    def close(self):
        """
        Close the socket connection.
        """
        self._sock.close()        
